package com.lyg.application.util;


import com.lyg.application.DTO.ErrorDto;
import com.lyg.application.entity.TaIncompatible;
import com.lyg.application.entity.TaRotation;
import com.lyg.application.mapper.TaIncompatibleMapper;
import com.lyg.application.mapper.TaRotationMapper;
import com.lyg.common.Constants;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

@Component
public class Rule {


    @Autowired
    TaIncompatibleMapper taIncompatibleMapper;
    @Autowired
    TaRotationMapper taRotationMapper;


    //这是在controller中使用的
    public Object taRotationRule(TaRotation taRotation) {
        Map<String, String> params = new Hashtable<>();
        String specificOrgName = taRotation.getSpecificOrgName();
        String specificNameOrgRotation = taRotation.getSpecificNameOrgRotation();
        String counterImpPositions = taRotation.getCounterImpPositions();
        String counterImpPostRotation = taRotation.getCounterImpPostRotation();

        if (specificOrgName != null && specificNameOrgRotation != null && counterImpPositions != null && counterImpPostRotation != null) {
            LocalDate newDate = LocalDate.now();
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            String passTime = newDate.format(formatter);
            params.put("oldOrgCode", taRotation.getSpecificOrgName());
            params.put("newOrgCode", taRotation.getSpecificNameOrgRotation());
            params.put("primaryPost", taRotation.getCounterImpPositions());
            params.put("targetPost", taRotation.getCounterImpPostRotation());
            params.put("passTime", passTime);


            List<ErrorDto> result = siftRule(params);

            return result;
        }
        return taRotation;
    }


    public List<ErrorDto> siftRule(Map<String, String> params) {
        String primaryPost = params.get("primaryPost");
        String targetPost = params.get("targetPost");

        List<TaRotation> repelList = getMonthRules(params);

        List<TaIncompatible> taIncompatibleList = regularRules(primaryPost, targetPost);

        RuleDetail ruleDetail=new RuleDetail();

        List<ErrorDto> result = ruleDetail.planRuleType(taIncompatibleList,repelList);


        return result;


    }


    public List<TaIncompatible> regularRules(String primaryPost, String targetPost) {
        return taIncompatibleMapper.getRegularRules(primaryPost, targetPost, Constants.REGULAR_RULE);

    }


    public List<TaRotation> getMonthRules(Map<String, String> params) {
        List<TaRotation> repelList = new ArrayList<>();

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        String primaryPost = params.get("primaryPost");
        String targetPost = params.get("targetPost");
        String passTime = params.get("passTime");
        LocalDate newDate = LocalDate.parse(passTime, formatter).minusMonths(3);

        List<String> oldPostList = taIncompatibleMapper.getMonthRulesPrimary(primaryPost, Constants.MONTH_RULE);
        List<String> newPostList = taIncompatibleMapper.getMonthRulesPrimary(targetPost, Constants.MONTH_RULE);

        if (oldPostList.size() == 0 && newPostList.size() == 0) {
            return repelList;
        } else {
            List<String> orgList = newPostList;//原来岗位+目标不相容岗位
            orgList.add(primaryPost);

            List<String> targetList = oldPostList;//目标岗位+原来不相容岗位
            targetList.add(targetPost);

            //拿到数据 不相容轮岗数据
            repelList = taRotationMapper.selectRepel(orgList, targetList, newDate,params);

            return repelList;
        }
    }
}

